package com.fs.course.service.impl;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fs.common.BeanCopyUtils;
import com.fs.common.exception.ServiceException;
import com.fs.common.utils.bean.BeanUtils;
import com.fs.course.domain.FsUserCoursePeriod;
import com.fs.course.dto.FsUserCourseTrainingCampDTO;
import com.fs.course.mapper.FsUserCoursePeriodMapper;
import com.fs.course.vo.FsUserCourseTrainingCampVO;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import com.fs.course.mapper.FsUserCourseTrainingCampMapper;
import com.fs.course.domain.FsUserCourseTrainingCamp;
import com.fs.course.service.IFsUserCourseTrainingCampService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;

/**
 * 会员训练营Service业务层处理
 *
 * @author fs
 * @date 2025-04-15
 */
@Service
@AllArgsConstructor
public class FsUserCourseTrainingCampServiceImpl extends ServiceImpl<FsUserCourseTrainingCampMapper, FsUserCourseTrainingCamp> implements IFsUserCourseTrainingCampService
{

    private final FsUserCoursePeriodMapper fsUserCoursePeriodMapper;

    /**
     * 查询训练营列表
     * @param params    参数
     * @return  list
     */
    @Override
    public List<FsUserCourseTrainingCampVO> selectFsUserCourseTrainingCampVOListByMap(Map<String, Object> params) {
        List<FsUserCourseTrainingCampVO> voList = baseMapper.selectFsUserCourseTrainingCampVOListByMap(params);

        // TODO: 会员数处理
        voList.forEach(v -> v.setVipCount(0));
        return voList;
    }

    /**
     * 新增训练营
     * @param params    参数
     */
    @Override
    public void add(FsUserCourseTrainingCampDTO params) {
        FsUserCourseTrainingCamp trainingCamp = new FsUserCourseTrainingCamp();
        trainingCamp.setTrainingCampName(params.getTrainingCampName());
        trainingCamp.setOrderNumber(baseMapper.getOrderNumber());
        trainingCamp.setCreateTime(LocalDateTime.now());
        baseMapper.insert(trainingCamp);
    }

    /**
     * 删除训练营
     * @param ids   ids
     */
    @Override
    public void del(Long[] ids) {
        baseMapper.deleteBatchIds(Arrays.asList(ids));
    }

    /**
     * 复制训练营
     * @param id    id
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void copy(Long id) {
        FsUserCourseTrainingCamp fsUserCourseTrainingCamp = baseMapper.selectById(id);
        if (Objects.isNull(fsUserCourseTrainingCamp)) {
            throw new ServiceException("训练营不存在");
        }

        // 复制训练营
        FsUserCourseTrainingCamp trainingCamp = new FsUserCourseTrainingCamp();
        trainingCamp.setTrainingCampName(fsUserCourseTrainingCamp.getTrainingCampName() + " - 副本");
        trainingCamp.setOrderNumber(baseMapper.getOrderNumber());
        trainingCamp.setCreateTime(LocalDateTime.now());
        baseMapper.insert(trainingCamp);

        // 复制营期
        List<FsUserCoursePeriod> fsUserCoursePeriods = fsUserCoursePeriodMapper.selectFsUserCoursePeriodListByTrainingCampId(id);
        if (!fsUserCoursePeriods.isEmpty()) {
            fsUserCoursePeriods.stream().map(p -> {
                FsUserCoursePeriod copy = new FsUserCoursePeriod();
                BeanUtils.copyProperties(p, copy, "periodId");
                copy.setTrainingCampId(trainingCamp.getTrainingCampId());
                return copy;
            }).forEach(fsUserCoursePeriodMapper::insertFsUserCoursePeriod);
        }
    }
}
